@@ -9,7 +9,7 @@ use strict;
 use warnings;
 
 BEGIN {
-   our $VERSION = '0.61_001';
+   our $VERSION = '0.61_002';
 }
 
 use Carp;
@@ -433,6 +433,14 @@ C<teardown_term>.
 
 =cut
 
+sub _flush
+{
+   my $self = shift;
+
+   $self->_flush_later if @{ $self->{todo_queue} };
+   $self->rootwin->flush if $self->rootwin;
+}
+
 sub _tick
 {
    my $self = shift;
@@ -451,7 +459,7 @@ sub _tick
       shift( @$timer_queue )->code->();
    }
 
-   $self->_flush_later if @{ $self->{todo_queue} };
+   $self->_flush;
 }
 
 sub tick
@@ -513,7 +521,7 @@ sub run
       die @_;
    };
 
-   $self->_flush_later if @{ $self->{todo_queue} };
+   $self->_flush;
 
    local $self->{keep_running} = 1;
    $self->_tick while( $self->{keep_running} );
