@@ -38,7 +38,7 @@ struct TickitWindow {
   bool focus_child_notify;
 
   int refcount;
-  struct TickitEventHook *hooks;
+  struct TickitHooklist hooks;
 };
 
 #define WINDOW_PRINTF_FMT     "[%dx%d abs@%d,%d]"
@@ -223,7 +223,7 @@ static void init_window(TickitWindow *win, TickitWindow *parent, TickitRect rect
   win->focus_child_notify = false;
 
   win->refcount = 1;
-  win->hooks = NULL;
+  win->hooks = (struct TickitHooklist){ NULL };
 }
 
 TickitWindow* tickit_window_new_root(TickitTerm *term)
@@ -341,7 +341,7 @@ void tickit_window_close(TickitWindow *win)
 
 void tickit_window_destroy(TickitWindow *win)
 {
-  tickit_hooklist_unbind_and_destroy(win->hooks, win);
+  tickit_hooklist_unbind_and_destroy(&win->hooks, win);
 
   if(win->pen)
     tickit_pen_unref(win->pen);
@@ -370,6 +370,9 @@ void tickit_window_destroy(TickitWindow *win)
     tickit_term_unref(root->term);
   }
 
+  DEBUG_LOGF("W*", "Window destroyed " WINDOW_PRINTF_FMT,
+      WINDOW_PRINTF_ARGS(win));
+
   free(win);
 }
 
@@ -570,6 +573,7 @@ static char *_gen_indent(TickitWindow *win)
     buf[0] = 0;
   }
 
+  buf[depth*2] = 0;
   for(char *s = buf; depth; depth--, s += 2)
     s[0] = '|', s[1] = ' ';
 
@@ -851,7 +855,8 @@ static void _do_hierarchy_change(HierarchyChangeType change, TickitWindow *paren
     DEBUG_LOGF("Wh", fmt,
         WINDOW_PRINTF_ARGS(parent), WINDOW_PRINTF_ARGS(win));
 
-  tickit_window_expose(parent, &win->rect);
+  if(win->is_visible)
+    tickit_window_expose(parent, &win->rect);
 }
 
 static void _request_hierarchy_change(HierarchyChangeType change, TickitWindow *win)
@@ -904,7 +909,8 @@ static bool _scrollrectset(TickitWindow *win, TickitRectSet *visible, int downwa
     if(!win->is_visible)
       return false;
 
-    tickit_pen_copy(pen, win->pen, false);
+    if(win->pen)
+      tickit_pen_copy(pen, win->pen, false);
 
     TickitWindow *parent = win->parent;
     if(!parent)
@@ -1172,27 +1178,38 @@ static int _handle_key(TickitWindow *win, TickitKeyEventInfo *info)
   if(!win->is_visible)
     return 0;
 
+  int ret = 1;
+  tickit_window_ref(win);
+
   if(win->first_child && win->first_child->steal_input)
     if(_handle_key(win->first_child, info))
-      return 1;
+      goto done;
 
   if(win->focused_child)
     if(_handle_key(win->focused_child, info))
-      return 1;
+      goto done;
 
   if(run_events_whilefalse(win, TICKIT_EV_KEY, info))
-    return 1;
+    goto done;
 
   // Last-ditch attempt to spread it around other children
-  for(TickitWindow *child = win->first_child; child; child = child->next) {
+  TickitWindow *next;
+  for(TickitWindow *child = win->first_child; child; child = next) {
+    next = child->next;
+
     if(child == win->focused_child)
       continue;
 
     if(_handle_key(child, info))
-      return 1;
+      goto done;
   }
 
-  return 0;
+  ret = 0;
+  /* fallthough */
+done:
+  tickit_window_unref(win);
+
+  return ret;
 }
 
 static TickitWindow *_handle_mouse(TickitWindow *win, TickitMouseEventInfo *info)
@@ -1200,7 +1217,13 @@ static TickitWindow *_handle_mouse(TickitWindow *win, TickitMouseEventInfo *info
   if(!win->is_visible)
     return NULL;
 
-  for(TickitWindow *child = win->first_child; child; child = child->next) {
+  TickitWindow *ret;
+  tickit_window_ref(win);
+
+  TickitWindow *next;
+  for(TickitWindow *child = win->first_child; child; child = next) {
+    next = child->next;
+
     int child_line = info->line - child->rect.top;
     int child_col  = info->col  - child->rect.left;
 
@@ -1215,13 +1238,18 @@ static TickitWindow *_handle_mouse(TickitWindow *win, TickitMouseEventInfo *info
     childinfo.line = child_line;
     childinfo.col  = child_col;
 
-    TickitWindow *ret;
     if((ret = _handle_mouse(child, &childinfo)))
-      return ret;
+      goto done;
   }
 
+  ret = win;
   if(run_events_whilefalse(win, TICKIT_EV_MOUSE, info))
-    return win;
+    goto done;
 
-  return 0;
+  ret = NULL;
+  /* fallthrough */
+done:
+  tickit_window_unref(win);
+
+  return ret;
 }
